/* Copyright (C) 2007 db4objects Inc. http://www.db4o.com */

package com.db4odoc.nqcollection;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.db4o.DatabaseFileLockedException;
import com.db4o.Db4o;
import com.db4o.ObjectContainer;
import com.db4o.ext.Db4oException;
import com.db4o.query.Predicate;

public class NQTests {

	private final static String DB4O_FILE_NAME = "reference.db4o";

	private final static int OBJECT_COUNT = 10;
	
	private static ObjectContainer _container = null;

	public static void main(String[] args) {
		storePilots();
		//selectTestPilots6PointsMore();
		//selectNonePilots();
		averagePilotPoints();
	}

	// end main

	private static ObjectContainer database() {
		if (_container == null) {
			try {
				_container = Db4o.openFile(DB4O_FILE_NAME);
			} catch (DatabaseFileLockedException ex) {
				System.out.println(ex.getMessage());
			}
		}
		return _container;
	}

	// end database
	
	private static void closeDatabase() {
		if (_container != null) {
			_container.close();
			_container = null;
		}
	}
	
	// end closeDatabase

	private static void storePilots() {
		new File(DB4O_FILE_NAME).delete();
		ObjectContainer container = database();
		if (container != null) {
			try {
				Pilot pilot;
				for (int i = 0; i < OBJECT_COUNT; i++) {
					pilot = new Pilot("Test Pilot #" + i, i);
					container.set(pilot);
				}
				for (int i = 0; i < OBJECT_COUNT; i++) {
					pilot = new Pilot("Professional Pilot #" + (i + 10), i + 10);
					container.set(pilot);
				}
				container.commit();
			} catch (Db4oException ex) {
				System.out.println("Db4o Exception: " + ex.getMessage());
			} catch (Exception ex) {
				System.out.println("System Exception: " + ex.getMessage());
			} finally {
				closeDatabase();
			}
		}
	}

	// end storePilots

	private static void selectNonePilots() {
		ObjectContainer container = database();
		if (container != null) {
			try {
				List<Pilot> result = container.query(new Predicate<Pilot>() {
					public boolean match(Pilot pilot) {
						// each Pilot is not included in the result
						return false;
					}
				});
				listResult(result);
			} catch (Exception ex) {
				System.out.println("System Exception: " + ex.getMessage());
			} finally {
				closeDatabase();
			}
		}
	}

	// end selectNonePilots

	private static void selectTestPilots6PointsMore() {
		ObjectContainer container = database();
		if (container != null) {
			try {
				List<Pilot> result = container.query(new Predicate<Pilot>() {
					public boolean match(Pilot pilot) {
						// all Pilots containing "Test" in the name
						// and 6 point are included in the result
						return ((pilot.getName().indexOf("Test") >= 0) && (pilot
								.getPoints() > 6));
					}
				});
				listResult(result);
			} catch (Exception ex) {
				System.out.println("System Exception: " + ex.getMessage());
			} finally {
				closeDatabase();
			}
		}
	}
	
	private static void averagePilotPoints() {
		ObjectContainer container = database();

		if (container != null) {
			try {
				AveragePredicate averagePredicate = new AveragePredicate();
				List<Pilot> result = container.query(averagePredicate);
				if (averagePredicate.count > 0) {
					System.out
							.println("Average points for professional pilots: "
									+ averagePredicate.sum
									/ averagePredicate.count);
				} else {
					System.out.println("No results");
				}
			} catch (Exception ex) {
				System.out.println("System Exception: " + ex.getMessage());
			} finally {
				closeDatabase();
			}
		}
	}

	// end averagePilotPoints

	private static class AveragePredicate extends Predicate<Pilot> {
		private int sum = 0;

		private int count = 0;

		public boolean match(Pilot pilot) {
			// return all pilots
			if (pilot.getName().startsWith("Professional")) {
				sum += pilot.getPoints();
				count++;
				return true;
			}
			return false;
		}
	}

	// end AveragePredicate


	private static void listResult(List result) {
		System.out.println(result.size());
		for (int i = 0; i < result.size(); i++) {
			System.out.println(result.get(i));
		}
	}

	// end listResult

	private static void listResult(Set result) {
		System.out.println(result.size());
		Iterator i = result.iterator();
		while (i.hasNext()) {
			System.out.println(i.next());
		}
	}
	// end listResult

}
