/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal.cs.messages;

import java.io.*;

import com.db4o.ext.*;
import com.db4o.foundation.*;
import com.db4o.internal.*;

/**
 * @exclude
 */
public class MCommittedInfo extends MsgD implements ClientSideMessage {

	public MCommittedInfo encode(CallbackObjectInfoCollections callbackInfo) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		
		encodeObjectInfoCollection(os, callbackInfo.added);
		encodeObjectInfoCollection(os, callbackInfo.deleted);
		encodeObjectInfoCollection(os, callbackInfo.updated);
		
		byte[] bytes = os.toByteArray();
		MCommittedInfo committedInfo = (MCommittedInfo) getWriterForLength(transaction(),
			bytes.length);
		committedInfo._payLoad.append(bytes);
		return committedInfo;
	}

	private void encodeObjectInfoCollection(ByteArrayOutputStream os, ObjectInfoCollection collection){
		Iterator4 iter = collection.iterator();
		while (iter.moveNext()) {
			LazyObjectReference obj = (LazyObjectReference) iter.current();
			writeLong(os, obj.getInternalID());
		}
		writeLong(os, -1);
	}

	public CallbackObjectInfoCollections decode() {
		CallbackObjectInfoCollections callbackInfo = CallbackObjectInfoCollections.EMTPY;
		ByteArrayInputStream is = new ByteArrayInputStream(_payLoad._buffer);
		callbackInfo.added = decodeObjectInfoCollection(is);
		callbackInfo.deleted = decodeObjectInfoCollection(is);
		callbackInfo.updated = decodeObjectInfoCollection(is);
		return callbackInfo;
	}

	private ObjectInfoCollection decodeObjectInfoCollection(ByteArrayInputStream is){
		final Collection4 collection = new Collection4();
		while (true) {
			long id = readLong(is);
			if (id == -1) {
				break;
			}
			collection.add(new LazyObjectReference(transaction(), (int) id));
		}
		return new ObjectInfoCollectionImpl(collection);
	}

	private void writeLong(ByteArrayOutputStream os, long l) {
		for (int i = 0; i < 64; i += 8) {
			os.write((int) (l >> i));
		}
	}

	private long readLong(ByteArrayInputStream is) {
		long l = 0;
		for (int i = 0; i < 64; i += 8) {
			l += ((long) (is.read())) << i;
		}
		return l;
	}

	public boolean processAtClient() {
		final CallbackObjectInfoCollections callbackInfos = decode();
		new Thread(new Runnable() {
			public void run() {
				if(stream().isClosed()){
					return;
				}
				stream().callbacks().commitOnCompleted(transaction(), callbackInfos);
			}
		}).start();
		return true;
	}

}
